@@ -76,7 +76,7 @@ class Area:
             yield coef
 
 
-@nb.njit("c16(c16,f8,f8[:,:])", cache=True)
+@nb.njit(cache=True)
 def log_evaluate_Nx(N, logx, area_list):
     r"""
     Evaluates a single logarithmic Lagrange interpolator in N-space multiplied
@@ -133,7 +133,7 @@ def log_evaluate_Nx(N, logx, area_list):
     return res
 
 
-@nb.njit("c16(c16,f8,f8[:,:])", cache=True)
+@nb.njit(cache=True)
 def evaluate_Nx(N, logx, area_list):
     r"""
     Evaluates a single linear Lagrange interpolator in N-space multiplied
@@ -176,11 +176,39 @@ def evaluate_Nx(N, logx, area_list):
     return res
 
 
+@nb.njit(cache=True)
+def evaluate_grid(N, is_log, logx, area_list):
+    """
+    Evaluate interpolator in N-space
+
+    Parameters
+    ----------
+        N : complex
+            Mellin variable
+        is_log : boolean
+            is a logarithmic interpolation
+        logx : float
+            logarithm of inversion point
+        area_list : list
+            area configuration of basis function
+
+    Returns
+    -------
+        pj : float
+            basis function * inversion factor
+    """ ""
+    if is_log:
+        pj = log_evaluate_Nx(N, logx, area_list)
+    else:
+        pj = evaluate_Nx(N, logx, area_list)
+    return pj
+
+
 # TODO lift to runcard?
 _atol_eps = 10 * np.finfo(float).eps
 
 
-@nb.njit("f8(f8,f8[:,:])", cache=True)
+@nb.njit(cache=True)
 def evaluate_x(x, area_list):
     """
     Get a single linear Lagrange interpolator in x-space
@@ -213,7 +241,7 @@ def evaluate_x(x, area_list):
     return res
 
 
-@nb.njit("f8(f8,f8[:,:])", cache=True)
+@nb.njit(cache=True)
 def log_evaluate_x(x, area_list):
     """
     Get a single logarithmic Lagrange interpolator in x-space
@@ -482,19 +510,15 @@ class InterpolatorDispatcher:
 
             *   - Name
                 - Type
-                - default
                 - description
             *   - ``interpolation_xgrid``
                 - :py:obj:`list(float)`
-                - [required]
                 - the interpolation grid
             *   - ``interpolation_polynomial_degree``
                 - :py:obj:`int`
-                - ``4``
                 - polynomial degree of the interpolating function
             *   - ``interpolation_is_log``
                 - :py:obj:`bool`
-                - ``True``
                 - use logarithmic interpolation?
 
         Parameters
@@ -538,21 +562,23 @@ class InterpolatorDispatcher:
         return self.basis[item]
 
     def get_interpolation(self, targetgrid):
-        """
-        Computes interpolation matrix between `targetgrid` and `xgrid`.
+        """Computes interpolation matrix between `targetgrid` and `xgrid`.
 
         .. math::
             f(targetgrid) = R \\cdot f(xgrid)
 
         Parameters
         ----------
-            targetgrid : array
-                grid to interpolate to
+        targetgrid : array
+            grid to interpolate to
 
         Returns
         -------
-            R : array
-                interpolation matrix, do be multiplied from the left(!)
+        R : array
+            interpolation matrix $R_{ij}$, where $i$ is the index over
+            `targetgrid`, and $j$ is the index on the internal basis (the
+            one stored in the :class:`InterpolatorDispatcher` instance)
+
         """
         # trivial?
         if len(targetgrid) == len(self.xgrid_raw) and np.allclose(
